# Settings for building Swift/T

#+======================+
#|| Mandatory settings ||
#+======================+
# Please fill in all of these variables

# Install root: change to your install location
export SWIFT_T_PREFIX=/tmp/exm-install

#+====================================+
#|| Optional configuration variables ||
#+====================================+
# These variables can be modified to enable/disable features or to
# provide information about your system and required software packages.
# For many systems the default settings are adequate.

# C compiler and flags
export CC=mpicc
# export CFLAGS=
# export LDFLAGS=

# Location of Tcl to override autodetected Tcl
# TCL_INSTALL=${EXM_PREFIX}/tcl
# Tcl version: specify in case of multiple competing versions
# TCL_VERSION=8.6

# Specify details of non-standard Tcl layouts
# name of tclsh compatible with build system
# TCLSH_LOCAL=/usr/bin/tclsh
# directory containing Tcl library
# TCL_LIB_DIR=/path/to/tcl/lib
# directory containing tcl.h
# TCL_INCLUDE_DIR=/path/to/tcl/include
# tcl system library directory.  Must contain init.tcl in root or
# tcl${TCL_VERSION} subdirectory
# TCL_SYSLIB_DIR=/path/to/tcl/lib

# Location of MPI install to override autodetected location
# MPI_INSTALL=${EXM_PREFIX}/mpi
# MPE_INSTALL=${EXM_PREFIX}/mpe
# Set MPI version
# (version 3 required for parallel tasks but version 2 is also allowed)
MPI_VERSION=3

# MPI custom layouts (uncomment below options to enable)
# EXM_CUSTOM_MPI=1
# MPI_INCLUDE=/path/to/mpi/include
# MPI_LIB_DIR=/path/to/mpi/lib
# MPI_LIB_NAME=funny.mpi.a

# Optimized build
# EXM_OPT_BUILD=1

# Debug build (extra logging)
# EXM_DEBUG_BUILD=1

# Enable Python integration
# ENABLE_PYTHON=1
# PYTHON_INSTALL=/usr/

# Enable R integration
# ENABLE_R=1
# R_INSTALL=/usr/

# Enable Julia integration
# ENABLE_JULIA=1
# JULIA_INSTALL=/usr/

# Enable task execution through Coasters C interface
# COASTER_INSTALL=/path/to/coaster-c-client

# Install location for Swift/K (for combined T & K package)
# SWIFTK_INSTALL=${EXM_PREFIX}/swift-k

# Enable MPE support
# ENABLE_MPE=1

# Disable ADLB checkpoint functionality
# DISABLE_XPT=1

# Manual zlib setup: required for checkpointing
# DISABLE_ZLIB=1
# ZLIB_INSTALL=/path/to/zlib

# Enable developer mode (extra warnings)
# EXM_DEV=1

# Disable build of static libraries and executables
# DISABLE_STATIC=1

# Do full static build (default is dynamic)
# EXM_STATIC_BUILD=1

# HDF5 location (yes/no/PATH location of h5cc)
# WITH_HDF5=no

#+================================+
#| Default build behavior         |
#+================================+
# These options control the default behavior of each build script, e.g.
# the extent to which it does a rebuild from scratch. These variables are
# overridden when you run other scripts like rebuild_all.sh.

# How to call Ant build tool
ANT=ant

# Ant arguments for STC build
STC_ANT_ARGS=""

# Make build parallelism: increase to speed up builds (make -j)
MAKE_PARALLELISM=4

# Run auto-tools before each build
RUN_AUTOTOOLS=${RUN_AUTOTOOLS:-0}

# Update from SVN in each source directory
SVN_UPDATE=${SVN_UPDATE:-0}

# Do a "make clean" or "ant clean"
MAKE_CLEAN=${MAKE_CLEAN:-0}

# Run configure
CONFIGURE=${CONFIGURE:-1}

#+=====================================+
#|| Optional directory layout control ||
#+=====================================+
# Specify non-standard source/install subdirectory locations

# Code: work out current directory
SCRIPT_DIR=$(cd $(dirname $0); pwd)

# Root of source directory. Default is current directory.
# Override with EXM_SRC_ROOT environment variable
EXM_SRC_ROOT=${EXM_SRC_ROOT:-$(${SCRIPT_DIR}/locate-src-root.sh ${SCRIPT_DIR} 3)}

# Source subdirectories. Modify to match your layout if needed
C_UTILS_SRC=${EXM_SRC_ROOT}/c-utils/code
LB_SRC=${EXM_SRC_ROOT}/lb/code
TURBINE_SRC=${EXM_SRC_ROOT}/turbine/code
STC_SRC=${EXM_SRC_ROOT}/stc/code

# Source subdirectory for external dependencies, if desired
#COASTER_SRC=

# Install subdirectories. Modify to match your layout if needed
LB_INSTALL=${SWIFT_T_PREFIX}/lb
C_UTILS_INSTALL=${SWIFT_T_PREFIX}/c-utils
TURBINE_INSTALL=${SWIFT_T_PREFIX}/turbine
STC_INSTALL=${SWIFT_T_PREFIX}/stc

#+======================================+
#|| Developer options (debugging, etc) ||
#+======================================+
# Verify integrity of bundled data
# ENABLE_MKSTATIC_CRC=1

# Local Variables:
# mode: sh
# End:
